"""This file is generated by setuptools-generate.
The information comes from pyproject.toml.
It provide some metainfo for docs/conf.py to build documents and
help2man to build man pages.
"""

# For docs/conf.py
project = "{{ data['project']['name'] }}"
author = """{% for author in data['project']['authors'] -%}
{{ author['name'] }} <{{ author['email'] }}> {% endfor -%}
"""
copyright = "{{ year }}"

# For help2man
DESCRIPTION = "{{ data['project']['description'] }}"
EPILOG = "Report bugs to {{ data['project']['urls']['Bug Report'] }}"
# format __version__ by yourself
VERSION = """{{ data['project']['name'] }} {__version__}
Copyright (C) {{ year }}
Written by {% for author in data['project']['authors'] -%}
{{ author['name'] }} <{{ author['email'] }}> {% endfor %}"""
SOURCE = "{{ data['project']['urls']['Source'] }}"
